/* ### CHECK-ME-BEFORE-RELEASE ### */
/*
 * Title:   mycvsfilter.c
 * Created: Thu Feb 10 21:52:11 2000
 * Authors: Tim Weyrich
 *     $Id: mycvsfilter.c,v 1.3 2003/11/19 10:37:29 weyrich Exp $
 *
 * Copyright (c) 2000--2003
 * Gilbert Baumann <unk6@stud.uni-karlsruhe.de>
 * Markus Noga <markus@noga.de>
 * Tim Weyrich <weyrich@inf.ethz.ch>
 *
 * This file is part of the Pointshop3D system.
 * See http: *www.pointshop3d.com/ for more information.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 * 
 * Contact info@pointshop3d.com if any conditions of this
 * licensing are not clear to you.
 *
 * ---------------------------------------------------------------
 *
 * $Log: mycvsfilter.c,v $
 * Revision 1.3  2003/11/19 10:37:29  weyrich
 * added Pointshop3D headers
 *
 * Revision 1.2  2002/02/28 15:31:17  weyrich
 * support for Mac files
 *
 * Revision 1.1.1.1  2001/10/09 16:26:23  Administrator
 * common compilation framework
 *
 * Revision 1.8  2000/06/30 15:48:11  tim
 * fix: superfluous free() in mycvsfilter
 *
 * Revision 1.7  2000/06/29 16:24:38  tim
 * mycvsclean compiles again on SGIs
 *
 * Revision 1.6  2000/05/14 10:17:55  tim
 * fix: now works again with stdout
 *
 * Revision 1.5  2000/03/18 02:50:17  tim
 * ANSI stuff
 *
 * Revision 1.4  2000/02/25 16:56:24  weyrich
 * codegeberation works also under VC++
 * renamed files to avoid VC++ name-clashes
 *
 * Revision 1.3  2000/02/23  22:00:34  weyrich
 * created header-files in anystream/include for each engine's lib
 *
 * Revision 1.2  2000/02/18  22:47:18  weyrich
 * fix: line truncation in Mac-files (cr-only)
 *
 * Revision 1.1  2000/02/14  22:15:02  weyrich
 * Q2 PVS finished
 * Renderer is now a C++-object
 * mycvsfilter works (say "make cvsclean")
 * tidied up bitvec.c and other files...
 *
 * Revision 1.1  2000/02/10  21:42:43  weyrich
 * added ourcvswrapper.c to anystream/util
 *
 */

#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <utime.h>
#include <string.h>
#include <errno.h>
#include <malloc.h>

char *mycvsfilter_strdup(const char *s)
{
  size_t len = strlen(s)+1;
  char *result = malloc(len);
  if (result)
    memcpy(result,s,len);
  return result;
}

#ifdef WIN32
#  define  PATHSEP        '\\'
#  define  PATHSEPSTRING  "\\"
#else
#  define  PATHSEP        '/'
#  define  PATHSEPSTRING  "/"
#  ifndef O_BINARY
#    define  O_BINARY  0
#  endif
#endif

/* KNOWN BUG:
 *
 *   stripping line ends only works, if tabbing mode is not LEAVE_TABS
 *
 */


#define  DEFAULT_TABSTYLE    UNTABIFY
#define  DEFAULT_STRIPSTYLE  STRIP_ENDS

typedef enum
{
  LEAVE_TABS,
  TABIFY,
  UNTABIFY
}
tabstyle_t;

const char  *taboption = "ltu";

tabstyle_t  tabstyle = DEFAULT_TABSTYLE;

int         tabsize = 8;


typedef enum
{
  STRIP_ENDS,
  DONT_STRIP
}
stripstyle_t;

stripstyle_t  stripstyle = DEFAULT_STRIPSTYLE;

const char  *stripoption = "sd";


void  filter (FILE *in, FILE *out)
{
  int  c=0, col=0;              /* current character and column */
  int  span_start=-1, span_len=0; /* current span of white-spaces */
  int  quoted_p = 0;            /* previous character was a back-slash */
  int  squote_p = 0;            /* we are in single-quotes */
  int  dquote_p = 0;            /* we are in double-quotes */
  int  quote_next_p = 0;        /* a back-slash was encountered */

  do
    {
      quoted_p = quote_next_p;
      quote_next_p = 0;

      c = fgetc(in);

      /* ignore all carriage-returns, followed by an new-line */
      if (c == '\r')
        if ((c = fgetc(in)) != '\n')
          {
            ungetc(c,in);
            c = '\n';
          }
	else
	  { /* Mac! */
	    c = '\n';
	  }

      if ((c == '\t' || c == ' ') &&
           !(quoted_p || squote_p || dquote_p))
        { /* white-spaces to tabify/untabify */
          int  n;
          if (c == '\t')
            n = tabsize - (col % tabsize);
          else
            n = 1;

          if (tabstyle == LEAVE_TABS)
            fputc(c, out);
          else
            {
              if (!span_len)  span_start = col;
              span_len += n;
            }
          col += n;
        }
      else
        { /* all other characters */
          switch (c)
            {
            case '\n':
              if (stripstyle == STRIP_ENDS)
                span_len = 0;
              break;

            case '"':
              if (!(squote_p || quoted_p))
                dquote_p = !dquote_p;
              break;

            case '\'':
              if (!(dquote_p || quoted_p))
                squote_p = !squote_p;
              break;

            case '\\':
              if (!quoted_p)
                quote_next_p = 1;
              break;
            }

          /* eventually finish a hanging span */
          if (span_len > 0) {
            if (tabstyle == UNTABIFY)
              fprintf (out, "%*c", span_len, ' ');
            else if (tabstyle == TABIFY)
              {
                int  n;
                for(;;)
                  {
                    n = tabsize - (span_start % tabsize);
                    if (n > span_len || n == 1)  break;
                    fputc ('\t', out);
                    span_len -= n;
                    span_start += n;
                  }
                if (span_len > 0)
                  fprintf (out, "%*c", span_len, ' ');
              }
          }
          span_len = 0;

          /* end of file? */
          if (c == EOF)  break;

          /* write through */
          fputc (c, out);
          if (c == '\n')
            col = 0;
          else
            col++;
        }
    }
  while (c != EOF);
}

int  copy (const char *src, const char *dest)
{
  ssize_t  n = -1;
  int  in_fd, out_fd;

  if ((in_fd = open(src, O_RDONLY|O_BINARY)) >= 0) {
    if ((out_fd = open(dest, O_CREAT|O_WRONLY|O_BINARY, 0666)) >= 0) {
      char     *buf[8192];
      ssize_t  nread, left;
      do {
        if ((n = read(in_fd, buf, 8192)) <= 0)  break;
        left = nread = n;
        do {
          n = write(out_fd, buf+nread-left, left);
          if (n <= 0)  break;
          left -= n;
        } while (left > 0);
      } while (n > 0);
      close (out_fd);
    }
    close (in_fd);
  }
  return n < 0;
}

int  seperate_path (const char *path, char **dir, char **fname)
{
  const char  *sep = strrchr (path, PATHSEP);
  if (sep) {
    *dir = (char *)malloc(sep-path+1);
    strncpy (*dir, path, sep-path);
    (*dir)[sep-path] = 0;
    *fname = mycvsfilter_strdup (sep+1);
  } else {
    *dir = mycvsfilter_strdup("./");
    *fname = mycvsfilter_strdup (path);
  }
  return 0;
}

char  *my_tempnam (const char *dir, const char *pfx)
{
  struct stat  st;
  const char   *dot;
  int          i, n;
  char         *p;
  const char   *ext;
  char         *path = (char *)malloc(strlen(dir)+20);
  const char   *sep = (dir[strlen(dir)-1] == PATHSEP) ? "" : PATHSEPSTRING;

  dot = strchr(pfx, '.');

  n = dot ? dot - pfx : strlen(pfx);
  if (n > 5)  n = 5;

  p = (char *)malloc (n+1);
  strncpy(p, pfx, n);
  p[n] = 0;

  ext = (dot && strlen(dot)<=4) ? dot : ".tmp";

  for (i=0; i<1000; i++)
    {
      sprintf (path, "%s%s%s%03d%s", dir, sep, p, i, ext);
      if (stat(path, &st))
        break;
    }

  free (p);

  if (errno == ENOENT)
    return path;
  else
    {
      free (path);
      return NULL;
    }
}

const char  *myself;

void  usage(FILE *out)
{
  fprintf (out, "usage: %s [-tulsd] [-z <tab-size>] [in-file] [out-file]\n", myself);
  fprintf (out, "           -%c  tabify\n", taboption[TABIFY]);
  fprintf (out, "           -%c  untabify\n", taboption[UNTABIFY]);
  fprintf (out, "           -%c  leave tabs as they are\n", taboption[LEAVE_TABS]);
  fprintf (out, "           -%c  strip trailing spaces in a line\n", stripoption[STRIP_ENDS]);
  fprintf (out, "           -%c  don't strip spaces\n", stripoption[DONT_STRIP]);
  fprintf (out, "Default options, that may be overridden: -%c -%c\n",
           taboption[tabstyle], stripoption[stripstyle]);
  fprintf (out, "The tab-size defaults to %d.\n", tabsize);
  fprintf (out, "Known Bug: `stripping line ends' only works, if\n");
  fprintf (out, "           tabbing mode is not LEAVE_TABS.\n");
}

int  main(int argc, const char *argv[])
{
  int         i, j;

  const char  *fname[2] = {NULL, NULL};
  int         fname_idx = 0;

  /* I use to do it the more humorous programming: */
  if ((myself = strrchr(argv[0], '/')+1) == ((const char *)NULL)+1)
    if ((myself = strrchr(argv[0], '\\')+1) == ((const char *)NULL)+1)
      myself = argv[0];

  for (i=1; i<argc; i++)
    {
      if (argv[i][0] == '-')
        if (!argv[i][1])
          fname_idx++;
        else
          for (j=1; j<strlen(argv[i]); j++) {
            if (argv[i][j] == taboption[TABIFY])
              tabstyle = TABIFY;
            else if (argv[i][j] == taboption[UNTABIFY])
              tabstyle = UNTABIFY;
            else if (argv[i][j] == taboption[LEAVE_TABS])
              tabstyle = LEAVE_TABS;
            else if (argv[i][j] == stripoption[STRIP_ENDS])
              stripstyle = STRIP_ENDS;
            else if (argv[i][j] == stripoption[DONT_STRIP])
              stripstyle = DONT_STRIP;
            else if (argv[i][j] == 'z')
	      {
		if (i+1 < argc && argv[i][j+1] == 0)
		  tabsize = atoi(argv[++i]);
		else
		  { usage(stderr); exit(1); }
	      }
            else if (argv[i][j] == 'h' ||
                     argv[i][j] == '?')
              { usage(stdout); exit(0); }
            else
              { usage(stderr); exit(1); }
          }
      else if (fname_idx < 2)
        fname[fname_idx++] = argv[i];
    }

  {
    int          in_place_p = (fname_idx == 1 && fname[0]);
    char         *inname = fname[0] ? mycvsfilter_strdup(fname[0]) : NULL;

    FILE         *in, *out;
    struct stat  instat;

    /* stat input file */
    if (fname[0] && stat(fname[0], &instat)) {
      fprintf (stderr, "couldn't stat %s: %s\n", fname[0], strerror(errno));
      exit (1);
    }

    /* organize temporary file */
    if (in_place_p) {
      char  *dir, *nam;
      seperate_path (fname[0], &dir, &nam);
      if (!(inname = my_tempnam (dir, nam))) {
        fprintf (stderr, "couldn't create temporary file -- aborting\n");
        exit (1);
      }
      /* copy input file to temp file */
      if (copy(fname[0], inname)) {
        fprintf (stderr, "couldn't copy %s to temporary file %s: %s\n",
                 fname[0], inname, strerror(errno));
      }
      fname[1] = fname[0];
    }

    /* open input (or temp) file */
    if (inname) {
      if (!(in = fopen (inname, "r"))) {
        fprintf (stderr, "couldn't open %s for read: %s\n", inname, strerror(errno));
        exit (1);
      }
    } else
      in = stdin;

    /* open output file */
    if (fname[1]) {
      if (!(out = fopen (fname[1], "wb"))) /* `b' for WIN32 */ {
	fprintf (stderr, "couldn't open %s for write: %s\n", fname[1], strerror(errno));
	exit (1);
      }
    } else
      out = stdout;

    /* filter text */
    filter(in, out);

    /* close files */
    if (inname)   fclose (out);
    if (fname[1]) fclose (in);
    
    /* remove temporary file */
    if (in_place_p)
      remove (inname);
    if (inname) free (inname);
    
    /* reset stat of output file */
    if (in_place_p) {
      struct utimbuf  utim;
      utim.actime  = instat.st_atime;
      utim.modtime = instat.st_mtime;
      if (utime (fname[1], &utim)) {
	fprintf (stderr, "couldn't utime %s: %s\n", fname[1], strerror(errno));
	exit (1);
      }
    }
  }
  
  exit (0);
  return -1;
}
