/* ### CHECK-ME-BEFORE-RELEASE ### */
/*
 * Title:   pisound.cpp
 * Created: Spring 2000
 * Authors: Markus Noga
 *     $Id: pisound.cpp,v 1.2 2003/11/19 10:37:30 weyrich Exp $
 *
 * Copyright (c) 2000--2003
 * Gilbert Baumann <unk6@stud.uni-karlsruhe.de>
 * Markus Noga <markus@noga.de>
 * Tim Weyrich <weyrich@inf.ethz.ch>
 *
 * This file is part of the Pointshop3D system.
 * See http: *www.pointshop3d.com/ for more information.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 * Contact info@pointshop3d.com if any conditions of this
 * licensing are not clear to you.
 *
 * ---------------------------------------------------------------
 *
 * $Log: pisound.cpp,v $
 * Revision 1.2  2003/11/19 10:37:30  weyrich
 * added Pointshop3D headers
 *
 *
 */

#include "pisound.h"

//! Create an IfSound object.
//
IfSound::IfSound(const char *dev,
               uint32 ra, uint32 bi, uint32 ch,
               uint32 bufsz, uint32 bufs)
{
  valid     =false;

  deviceName=dev? strdup(dev) : NULL;
  rate      =ra;
  bits      =bi;
  channels  =ch;
  sampleSize=ch*bi/8;
  bufSize   =bufsz;
  buffers   =bufs;

  // validate sample size
  //
  if(bits!=8 && bits!=16) {
    genError("set format: only 8/16 bit supported");
    return;
  }

  // validate channels
  //
  if(channels!=1 && channels!=2) {
    genError("set format: only 1/2 channels supported.");
    return;
  }

  // validate buffering
  //
  if(buffers==0 || bufSize==0) {
    genError("set buffering: minimum 1 buffer of 1 sample.");
    return;
  }

  valid =true;
  errMsg=NULL;
}

IfSound::~IfSound() {
  if(deviceName)
    free(deviceName);
  if(errMsg)
    delete errMsg;
}

void IfSound::genError(const char *msg) {
  uint32 len=strlen(msg)+8;

  errMsg=new char[len];
  if(errMsg)
    sprintf(errMsg,"sound: %s",msg);
}


