/* ### CHECK-ME-BEFORE-RELEASE ### */
/*
 * Title:   win16sound.cpp
 * Created: Spring 2000
 * Authors: Markus Noga
 *     $Id: win16sound.cpp,v 1.3 2003/11/19 10:37:31 weyrich Exp $
 *
 * Copyright (c) 2000--2003
 * Gilbert Baumann <unk6@stud.uni-karlsruhe.de>
 * Markus Noga <markus@noga.de>
 * Tim Weyrich <weyrich@inf.ethz.ch>
 *
 * This file is part of the Pointshop3D system.
 * See http: *www.pointshop3d.com/ for more information.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 * 
 * Contact info@pointshop3d.com if any conditions of this
 * licensing are not clear to you.
 *
 * ---------------------------------------------------------------
 *
 * $Log: win16sound.cpp,v $
 * Revision 1.3  2003/11/19 10:37:31  weyrich
 * added Pointshop3D headers
 *
 *
 */

#include <pisound.h>
#include <mmreg.h>
#include <mmsystem.h>

#define DO_DEBUG


#ifdef DO_DEBUG
# include <win32funcs.h>
#endif


//! Sound thread
static void *win32sound_thread(void *arg) {
  Win32Sound *snd=(Win32Sound*) arg;

  fprintf(stderr,"soundthread: starting\n");

  while(1) {
    MSG msg;
    BOOL res;

    res=GetMessage(&msg,NULL,0,0);
    if(res==0 || res==-1)
      break;

    if(msg.message==MM_WOM_DONE)
      snd->renderAndPlayBuffer((WAVEHDR*) msg.lParam);

#ifdef DO_DEBUG    
    else {
      // report unknown event
      //
      const winevent_t *evt=winevent;
      while(evt->value!=msg.message && evt->name)
         ;
      fprintf(stderr,"soundthread: message %x (%s)\n",
              msg.message, evt-> name ? evt->name : "unknown" );
    }
#endif

  }
  fprintf(stderr,"soundthread: terminating\n.");
  return (void*) 0;
}

static void CALLBACK win32sound_event(UINT wTimerID, UINT msg, 
				      DWORD dwUser, DWORD dw1, DWORD dw2) {
  Win32Sound *snd=(Win32Sound*) dwUser;
  snd->renderAndPlayNextBuffer();
}

static CALLBACK win32sound_callback(  HWAVEOUT hwo,        UINT uMsg,         
				    DWORD dwInstance,    DWORD dwParam1,      DWORD dwParam2     ) {
  if(uMsg==WOM_DONE) {
    Win32Sound *snd=(Win32Sound*) dwInstance;
    snd->renderAndPlayBuffer((WAVEHDR*) dwParam1);
  }
}

  
Win32Sound::Win32Sound(const char *device,
        	       uint32 rate, uint32 bits, uint32 channels,
        	       uint32 bufsize, uint32 buffers)
  : IfSound(device,rate,bits,channels,bufsize,buffers)
{
  UINT deviceID=WAVE_MAPPER;    // FIXME: device string discarded.
  WAVEFORMATEX fmt;
  MMRESULT res;
  uint32 i;

  // check superclass initialization validity
  //
  if(!isValid())
    return;
  valid=false;
  
  // create playback thread  
  //
  pth=PiThread::create(win32sound_thread,(void*) this);
  if(!pth) {
    IfSound::genError("creating thread");
    return;
  }
  SetThreadPriority(pth->getPlatformHandle(),THREAD_PRIORITY_ABOVE_NORMAL);

  // prepare MMSYSTEM sample format header
  //
  fmt.wFormatTag     =WAVE_FORMAT_PCM;
  fmt.nSamplesPerSec =rate;
  fmt.wBitsPerSample =(WORD) bits;
  fmt.nChannels      =(WORD) channels;
  fmt.nBlockAlign    =(WORD) getSampleSize();
  fmt.nAvgBytesPerSec=rate*getSampleSize();
  fmt.cbSize         =0;

  // open wave output device
  //

  //uint32 devs=waveOutGetNumDevs();
  //fprintf(stderr,"%ld wave devices present.\n",devs);

  res=waveOutOpen(&wout, deviceID, &fmt, 
//		  NULL, NULL, 0);
//                  (DWORD) pth->getPlatformId(), (DWORD) this,
//                  CALLBACK_THREAD);
                  (DWORD) win32sound_callback, (DWORD) this,
                  CALLBACK_FUNCTION);
  
  if(res!=MMSYSERR_NOERROR) {
    genError("opening device",res);
    pth->terminate();
    delete pth;
    return;
  }
  
  // allocate and initialize data buffers
  //
  bufdata   =new uint8  [buffers*bufsize*getSampleSize()];
  if(!bufdata) {
    IfSound::genError("allocating memory");
    waveOutClose(wout);
    pth->terminate();
    delete pth;
    return;
  }
  bufheaders=new WAVEHDR[buffers];
  if(!bufheaders) {
    IfSound::genError("allocating memory");
    delete bufdata;
    waveOutClose(wout);
    pth->terminate();
    delete pth;
    return;
  }

  for(i=0; i<buffers; i++) {
    bufheaders[i].lpData         =(LPSTR) (bufdata + i*bufsize*getSampleSize());
    bufheaders[i].dwBufferLength =bufsize*getSampleSize();
    bufheaders[i].dwBytesRecorded=0;
    bufheaders[i].dwUser         =(DWORD) this;
    bufheaders[i].dwFlags        =0;
    bufheaders[i].dwLoops        =0;
    bufheaders[i].lpNext         =NULL;
    bufheaders[i].reserved       =0;

    waveOutPrepareHeader(wout,&(bufheaders[i]),sizeof(WAVEHDR));
  }

  valid=true;
}

Win32Sound::~Win32Sound() {
  if(isValid()) {
    waveOutReset(wout);

    pth->terminate();
    delete pth;

    delete bufdata;
    delete bufheaders;

    waveOutClose(wout);
  }
}

void Win32Sound::play() {
  if(isValid()) {
    uint32 i;
    for(i=0; i<buffers; i++)
      renderAndPlayBuffer(bufheaders+i);

/*    nextBuffer=0;
    fprintf(stderr,"this = %p\n",this);
    timerId=timeSetEvent(rate/bufSize-1,0,win32sound_event,(DWORD) this,
                         TIME_PERIODIC | TIME_CALLBACK_FUNCTION);
*/			 
  }
}

void Win32Sound::renderAndPlayBuffer(WAVEHDR *hdr) {
  if(isValid()) {
    renderBuffer((void*) hdr->lpData);
    waveOutWrite(wout,hdr,sizeof(WAVEHDR));
  }
}

void Win32Sound::renderAndPlayNextBuffer() {
  if(isValid()) {
    if(bufheaders[nextBuffer].dwFlags & WHDR_DONE) {
      renderAndPlayBuffer(&(bufheaders[nextBuffer]));
      nextBuffer=(nextBuffer+1)%buffers;
    }
  }
}


void Win32Sound::genError(const char *msg,MMRESULT nr) {
  if(nr) {
    char err[MAXERRORLENGTH];
    uint32 len;

    waveOutGetErrorText(nr,err,MAXERRORLENGTH);
    len=strlen(msg)+strlen(err)+10;

    errMsg=new char[len];
    if(errMsg)
      sprintf(errMsg,"sound: %s: %s",msg,err);
  } else
    IfSound::genError(msg);
}
