// ### CHECK-ME-BEFORE-RELEASE ###
//
// Title:   normaltest.cpp
// Created: Sun Apr 28 16:24:58 2002
// Authors: Tim Weyrich <weyrich@inf.ethz.ch>
//
// Copyright (c) 2001--2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//
// ---------------------------------------------------------------
//
// $Log: normaltest.cpp,v $
// Revision 1.2  2003/11/19 13:04:00  weyrich
// added Pointshop3D headers
//
// Revision 1.1.1.1  2002/05/08 08:58:48  weyrich
// surfel file format
//

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

#include <sfl/normalenc.h>

#define  NUM_TESTS  1000000
#define  TEST_MODE  CUBIC_16

int  main(void)
{
  normalenc_compression_enum  mode = TEST_MODE;

  int  i;
  double  min = 1e100;
  double  max = 0;

  fprintf(stderr, "Null vector test...");
  {
    double  x, y, z;
    if (normalenc_decode_normal_3d(mode, normalenc_encode_normal(mode, 0, 0, 0), &x, &y, &z))
      {
        fprintf(stderr, "ERROR ON DECODING (code was 0x%08x).\n",
                (unsigned)normalenc_encode_normal(mode, 0, 0, 0));
      }
    else
      fprintf(stderr, "%s.\n",
              (x*x + y*y + z*z == 0.0) ? "PASSED" : "FAILED");
  }

  for (i=0; i<NUM_TESTS; )
    {
      double  x, y, z, a;
      x = (2.0 * rand() / RAND_MAX) - 1.0;
      y = (2.0 * rand() / RAND_MAX) - 1.0;
      z = (2.0 * rand() / RAND_MAX) - 1.0;
      a = sqrt(x*x + y*y + z*z);
      x /= a;
      y /= a;
      z /= a;
      if (z > fabs(x) && z > fabs(y))
        {
          float u, v, w;
          uint32  code;
          code = normalenc_encode_normal(mode, x, y, z);
          if (normalenc_decode_normal_3f(mode, code, &u, &v, &w))
            {
              fprintf(stderr, "error on decoding: [%12.10g %12.10g %12.10g] -> 0x%04x\n",
                      x, y, z, (unsigned)code);
            }
          double  dx = x - u;
          double  dy = y - v;
          double  dz = z - w;
          a = sqrt(dx*dx + dy*dy + dz*dz);
          if (a > max)  max = a;
          if (a < min)  min = a;
          if (0 && a == max)
            {
              fprintf(stderr, "[%12.10g %12.10g %12.10g\n %12.10g %12.10g %12.10g]: %12.10g (0x%08x)\n",
                      x, y, z, u, v, w, a, (unsigned)code);
            }
          i++;
        }
    }
  fprintf(stderr, "min: %10.8f; max: %10.8f\n", min, max);
  return 0;
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
